Skip to content

This is most useful for generating a specification using the short form or coercing from a list.

Usage

as.col_spec(x, call = caller_env())

Arguments

x

Input object

Examples

as.col_spec("cccnnn")
#> cols(
#>   col_character(),
#>   col_character(),
#>   col_character(),
#>   col_number(),
#>   col_number(),
#>   col_number()
#> )