Learn R Programming

vroom (version 1.7.0)

as.col_spec: Coerce to a column specification

Description

This is most useful for generating a specification using the short form or coercing from a list.

Usage

as.col_spec(x, call = caller_env())

Arguments

x

Input object

Examples

Run this code
as.col_spec("cccnnn")

Run the code above in your browser using DataLab