- rows
Number of rows to generate
- cols
Number of columns to generate, if NULL this is derived from col_types.
- col_types
One of NULL, a cols() specification, or
a string.
If NULL, all column types will be inferred from guess_max rows
of the input, interspersed throughout the file. This is convenient (and
fast), but not robust. If the guessed types are wrong, you'll need to
increase guess_max or supply the correct types yourself.
Column specifications created by list() or cols() must contain
one column specification for each column. If you only want to read a
subset of the columns, use cols_only().
Alternatively, you can use a compact string representation where each
character represents one column:
c = character
i = integer
I = big integer
n = number
d = double
l = logical
f = factor
D = date
T = date time
t = time
? = guess
_ or - = skip
By default, reading a file without a column specification will print a
message showing the guessed types. To suppress this message, set
show_col_types = FALSE.
- locale
The locale controls defaults that vary from place to place.
The default locale is US-centric (like R), but you can use
locale() to create your own locale that controls things like
the default time zone, encoding, decimal mark, big mark, and day/month
names.
- missing
The percentage (from 0 to 1) of missing data to use